#include "lpc23xx.h"
#include "lcddrv.h"
#include "chartable.h"

/**
* Author: Steffen Graf
* Date: 15.11.09
* Version: 1.0
* Description: Inits the LCD Controller in 24 bit mode with 9 MHz pixel clock
**/ 

void TFT_init(){
	PINSEL0 |= 0x00055500;
	PINSEL3 |= 0x05555500;
	PINSEL4 |= 0x050FFFFF;
	PINSEL9 |= 0x0A000000;
	PINSEL11 |= 0x0000000F;

	PCONP |= (1<<20);		//Enable LCD controller
	LCD_CTRL = 0x2A;		//TFT 24 Bit
	LCD_CFG = 7;			//Set divider CCLK/8 = 9 MHz 
	LCD_POL =  (0x01 << 26)|((x_size-1) << 16)|(0x00 << 14)|(0x00 << 13)|(0x01 << 12)|(0x01 << 11);  
	//Bypass pixel clock divider | 800 clocks per line | invert: h clock, v clock
	LCD_TIMH =  (87 << 24)|(39 << 16)|(127 << 8 )|(((x_size/16)-1) << 2);  
	//Back porch = 88 clk | Front porch = 40 clk | Pulse with = 128 clk | Size = (49 + 1)*16 = 800 px
	LCD_TIMV = (22 << 24)|(1 << 16)|(3 << 10)|(y_size-1);      
	//Back porch = 23 px | Front porch = 2 px | Pulse with = 4 px | Size = 600 px
	LCD_UPBASE = (int) *FRAMEBUFFER;
	LCD_LPBASE = (int) *FRAMEBUFFER;
	LCD_CTRL |= 1|(1<<11);	//Enable LCD
}

/**
* Author: Steffen Graf
* Date: 16.11.09
* Version: 1.0
* Description: writes a character on given position
**/ 

void lcd_char(int x, int y, char c){
	int h, mask,p;
	char ch;
	
	unsigned int textcolor = 0x00ffffff;
	unsigned int backcolor = 0;
	
	for (h=0; h<CHAR_H; h++){ // every column of the character        
        ch=ascii_tab[ c-32 ][h];
        mask=0x80;
        for (p=0; p<CHAR_W; p++)  // write the pixels
        {
			if (ch&mask){
				FRAMEBUFFER[(y*CHAR_H)+h][(x*CHAR_W)+p] = textcolor;                
			}else{
				FRAMEBUFFER[(y*CHAR_H)+h][(x*CHAR_W)+p] = backcolor;
			}
			mask=mask/2;
        }
	}
}

/**
* Author: Steffen Graf
* Date: 16.11.09
* Version: 1.2
* Description: writes a char array on given position
**/ 

void lcd_str(int x, int y, char *str){ 
	int i = 0;
	while(str[i] != 0){
		if((str[i] == 10)){
			i++;
			y++;
			x=0;
		}else{
			if (str[i] != 13)
				lcd_char(x++, y, str[i++]);
			else
				i++;
		}
		if (x==100){
			x=0;
			y++;
		}
	}
}

/**
* Author: Steffen Graf
* Date: 16.11.09
* Version: 1.1
* Description: writes a positive int number on given position
**/

void lcd_int(int x, int y, int data){
	unsigned int byt;
	if (data<9){
		lcd_char(x++,y,data + '0');
		lcd_char(x++,y, ' ');
		lcd_char(x++,y, ' ');
		lcd_char(x,y, ' ');
	}
	
	if ((data<99) && (data>9)){
		byt = data / 10;
		data = data - byt * 10;
		lcd_char(x++,y,byt + '0');
		lcd_char(x++,y,data + '0');
		lcd_char(x++,y, ' ');
		lcd_char(x,y, ' '); 
    }
	
	if ((data<999) && (data>99)){
		byt = data / 100;
		data = data - byt * 100;
		lcd_char(x++,y,byt + '0');
		byt = data / 10;
		data = data - byt * 10;
		lcd_char(x++,y,byt + '0');
		lcd_char(x++,y,data + '0');
		lcd_char(x,y, ' ');
    }
	
	if ((data<9999) && (data>999)){
		byt = data / 1000;
		data = data - byt * 1000;
		lcd_char(x++,y,byt + '0');
		byt = data / 100;
		data = data - byt * 100;
		lcd_char(x++,y,byt + '0');
		byt = data / 10;
		data = data - byt * 10;
		lcd_char(x++,y,byt + '0');
		lcd_char(x++,y,data + '0');
		lcd_char(x,y, ' ');
    }
	
	if ((data<99999) && (data>9999)){
		byt = data / 10000;
		data = data - byt * 10000;
		lcd_char(x++,y,byt + '0');
		byt = data / 1000;
		data = data - byt * 1000;
		lcd_char(x++,y,byt + '0');
		byt = data / 100;
		data = data - byt * 100;
		lcd_char(x++,y,byt + '0');
		byt = data / 10;
		data = data - byt * 10;
		lcd_char(x++,y,byt + '0');
		lcd_char(x,y,data + '0');
    }
	
	if ((data<999999) && (data>99999)){
		byt = data / 100000;
		data = data - byt * 100000;
		lcd_char(x++,y,byt + '0');
		byt = data / 10000;
		data = data - byt * 10000;
		lcd_char(x++,y,byt + '0');
		byt = data / 1000;
		data = data - byt * 1000;
		lcd_char(x++,y,byt + '0');
		byt = data / 100;
		data = data - byt * 100;
		lcd_char(x++,y,byt + '0');
		byt = data / 10;
		data = data - byt * 10;
		lcd_char(x++,y,byt + '0');
		lcd_char(x,y,data + '0');
    }
	
	if ((data<9999999) && (data>999999)){
		byt = data / 1000000;
		data = data - byt * 1000000;
		lcd_char(x++,y,byt + '0');
		byt = data / 100000;
		data = data - byt * 100000;
		lcd_char(x++,y,byt + '0');
		byt = data / 10000;
		data = data - byt * 10000;
		lcd_char(x++,y,byt + '0');
		byt = data / 1000;
		data = data - byt * 1000;
		lcd_char(x++,y,byt + '0');
		byt = data / 100;
		data = data - byt * 100;
		lcd_char(x++,y,byt + '0');
		byt = data / 10;
		data = data - byt * 10;
		lcd_char(x++,y,byt + '0');
		lcd_char(x,y,data + '0');
    }
	
	if ((data<99999999) && (data>9999999)){
		byt = data / 10000000;
		data = data - byt * 10000000;
		lcd_char(x++,y,byt + '0');
		byt = data / 1000000;
		data = data - byt * 1000000;
		lcd_char(x++,y,byt + '0');
		byt = data / 100000;
		data = data - byt * 100000;
		lcd_char(x++,y,byt + '0');
		byt = data / 10000;
		data = data - byt * 10000;
		lcd_char(x++,y,byt + '0');
		byt = data / 1000;
		data = data - byt * 1000;
		lcd_char(x++,y,byt + '0');
		byt = data / 100;
		data = data - byt * 100;
		lcd_char(x++,y,byt + '0');
		byt = data / 10;
		data = data - byt * 10;
		lcd_char(x++,y,byt + '0');
		lcd_char(x,y,data + '0');
    }
	
	if ((data<999999999) && (data>99999999)){
		byt = data / 100000000;
		data = data - byt * 100000000;
		lcd_char(x++,y,byt + '0');
		byt = data / 10000000;
		data = data - byt * 10000000;
		lcd_char(x++,y,byt + '0');
		byt = data / 1000000;
		data = data - byt * 1000000;
		lcd_char(x++,y,byt + '0');
		byt = data / 100000;
		data = data - byt * 100000;
		lcd_char(x++,y,byt + '0');
		byt = data / 10000;
		data = data - byt * 10000;
		lcd_char(x++,y,byt + '0');
		byt = data / 1000;
		data = data - byt * 1000;
		lcd_char(x++,y,byt + '0');
		byt = data / 100;
		data = data - byt * 100;
		lcd_char(x++,y,byt + '0');
		byt = data / 10;
		data = data - byt * 10;
		lcd_char(x++,y,byt + '0');
		lcd_char(x,y,data + '0');
    }
	

}

/**
* Author: Steffen Graf
* Date: 21.11.09
* Version: 1.1
* Description: sets a point
**/

void lcd_pset(int x,int y, int color){
	FRAMEBUFFER[y][x] = color;
}
